"""
Type annotations for botocore.discovery module.

Copyright 2025 Vlad Emelianov
"""

from logging import Logger
from typing import Any, Callable, Mapping

from botocore.client import BaseClient
from botocore.exceptions import BotoCoreError as BotoCoreError
from botocore.exceptions import ConnectionError as ConnectionError
from botocore.exceptions import HTTPClientError as HTTPClientError
from botocore.hooks import BaseEventHooks
from botocore.model import OperationModel, ServiceModel
from botocore.model import OperationNotFoundError as OperationNotFoundError
from botocore.utils import CachedProperty as CachedProperty

logger: Logger = ...

class EndpointDiscoveryException(BotoCoreError): ...

class EndpointDiscoveryRequired(EndpointDiscoveryException):
    fmt: str = ...

class EndpointDiscoveryRefreshFailed(EndpointDiscoveryException):
    fmt: str = ...

def block_endpoint_discovery_required_operations(model: OperationModel, **kwargs: Any) -> None: ...

class EndpointDiscoveryModel:
    def __init__(self, service_model: ServiceModel) -> None: ...
    @CachedProperty
    def discovery_operation_name(self) -> str: ...
    @CachedProperty
    def discovery_operation_keys(self) -> list[str]: ...
    def discovery_required_for(self, operation_name: str) -> bool: ...
    def discovery_operation_kwargs(self, **kwargs: Any) -> dict[str, Any]: ...
    def gather_identifiers(
        self, operation: OperationModel, params: Mapping[str, Any]
    ) -> dict[str, Any]: ...

class EndpointDiscoveryManager:
    def __init__(
        self,
        client: BaseClient,
        cache: Any | None = ...,
        current_time: Callable[[], float] | None = ...,
        always_discover: bool = ...,
    ) -> None: ...
    def gather_identifiers(self, operation: OperationModel, params: Mapping[str, Any]) -> Any: ...
    def delete_endpoints(self, **kwargs: Any) -> None: ...
    def describe_endpoint(self, **kwargs: Any) -> Any: ...

class EndpointDiscoveryHandler:
    def __init__(self, manager: EndpointDiscoveryManager) -> None: ...
    def register(self, events: BaseEventHooks, service_id: str) -> None: ...
    def gather_identifiers(
        self,
        params: Mapping[str, Any],
        model: OperationModel,
        context: Mapping[str, Any],
        **kwargs: Any,
    ) -> None: ...
    def discover_endpoint(self, request: Any, operation_name: str, **kwargs: Any) -> None: ...
    def handle_retries(
        self,
        request_dict: Mapping[str, Any],
        response: Mapping[str, Any],
        operation: OperationModel,
        **kwargs: Any,
    ) -> Any: ...



"""
Type annotations for botocore.regions module.

Copyright 2025 Vlad Emelianov
"""

from collections.abc import Iterable
from enum import Enum
from logging import Logger
from typing import Any, Mapping

from botocore.auth import AUTH_TYPE_MAPS as AUTH_TYPE_MAPS
from botocore.compat import HAS_CRT as HAS_CRT
from botocore.crt import CRT_SUPPORTED_AUTH_TYPES as CRT_SUPPORTED_AUTH_TYPES
from botocore.endpoint_provider import RuleSetEndpoint
from botocore.exceptions import BotoCoreError
from botocore.exceptions import NoRegionError as NoRegionError
from botocore.hooks import BaseEventHooks
from botocore.model import OperationModel, ServiceModel

LOG: Logger = ...
DEFAULT_URI_TEMPLATE: str
DEFAULT_SERVICE_DATA: dict[str, dict[str, Any]]

class BaseEndpointResolver:
    def construct_endpoint(self, service_name: str, region_name: str | None = ...) -> None: ...
    def get_available_partitions(self) -> list[str]: ...
    def get_available_endpoints(
        self,
        service_name: str,
        partition_name: str = ...,
        allow_non_regional: bool = ...,
    ) -> list[str]: ...

class EndpointResolver(BaseEndpointResolver):
    def __init__(self, endpoint_data: Mapping[str, Any], uses_builtin_data: bool = ...) -> None: ...
    def get_service_endpoints_data(self, service_name: str, partition_name: str = ...) -> Any: ...
    def get_available_partitions(self) -> list[str]: ...
    def get_available_endpoints(
        self,
        service_name: str,
        partition_name: str = ...,
        allow_non_regional: bool = ...,
        endpoint_variant_tags: Iterable[str] | None = ...,
    ) -> list[str]: ...
    def get_partition_dns_suffix(
        self, partition_name: str, endpoint_variant_tags: Iterable[str] | None = ...
    ) -> str: ...
    def construct_endpoint(  # type: ignore [override]
        self,
        service_name: str,
        region_name: str | None = ...,
        partition_name: str | None = ...,
        use_dualstack_endpoint: bool = ...,
        use_fips_endpoint: bool = ...,
    ) -> dict[str, Any] | None: ...
    def get_partition_for_region(self, region_name: str) -> str: ...

class EndpointResolverBuiltins(Enum):
    AWS_REGION = "AWS::Region"
    AWS_USE_FIPS = "AWS::UseFIPS"
    AWS_USE_DUALSTACK = "AWS::UseDualStack"
    AWS_STS_USE_GLOBAL_ENDPOINT = "AWS::STS::UseGlobalEndpoint"
    AWS_S3_USE_GLOBAL_ENDPOINT = "AWS::S3::UseGlobalEndpoint"
    AWS_S3_ACCELERATE = "AWS::S3::Accelerate"
    AWS_S3_FORCE_PATH_STYLE = "AWS::S3::ForcePathStyle"
    AWS_S3_USE_ARN_REGION = "AWS::S3::UseArnRegion"
    AWS_S3CONTROL_USE_ARN_REGION = "AWS::S3Control::UseArnRegion"
    AWS_S3_DISABLE_MRAP = "AWS::S3::DisableMultiRegionAccessPoints"
    SDK_ENDPOINT = "SDK::Endpoint"
    ACCOUNT_ID = "AWS::Auth::AccountId"
    ACCOUNT_ID_ENDPOINT_MODE = "AWS::Auth::AccountIdEndpointMode"

class EndpointRulesetResolver:
    def __init__(
        self,
        endpoint_ruleset_data: Mapping[str, Any],
        partition_data: Mapping[str, Any],
        service_model: ServiceModel,
        builtins: EndpointResolverBuiltins,
        client_context: Mapping[str, Any],
        event_emitter: BaseEventHooks,
        use_ssl: bool = ...,
        requested_auth_scheme: str | None = ...,
    ) -> None: ...
    def construct_endpoint(
        self,
        operation_model: OperationModel,
        call_args: Mapping[str, Any] | None,
        request_context: Mapping[str, Any],
    ) -> RuleSetEndpoint: ...
    def auth_schemes_to_signing_ctx(
        self, auth_schemes: list[Mapping[str, Any]]
    ) -> tuple[str, dict[str, Any]]: ...
    def ruleset_error_to_botocore_exception(
        self, ruleset_exception: Exception, params: Mapping[str, Any]
    ) -> BotoCoreError: ...



"""
Type annotations for botocore.compress module.

Copyright 2025 Vlad Emelianov
"""

import io
from gzip import GzipFile as GzipFile
from logging import Logger
from typing import Any, Callable, Mapping

from botocore.compat import urlencode as urlencode
from botocore.config import Config
from botocore.model import OperationModel
from botocore.utils import determine_content_length as determine_content_length

logger: Logger = ...

def maybe_compress_request(
    config: Config, request_dict: Mapping[str, Any], operation_model: OperationModel
) -> None: ...

COMPRESSION_MAPPING: dict[str, Callable[[io.BytesIO], io.BytesIO]] = ...



"""
Type annotations for botocore.errorfactory module.

Copyright 2025 Vlad Emelianov
"""

from typing import Mapping

from botocore.exceptions import ClientError as ClientError
from botocore.exceptions import ClientError as _ClientError
from botocore.model import ServiceModel
from botocore.utils import get_service_module_name as get_service_module_name

class BaseClientExceptions:
    ClientError: type[_ClientError] = ...  # noqa: F811
    def __init__(self, code_to_exception: Mapping[str, type[_ClientError]]) -> None: ...
    def from_code(self, error_code: str) -> _ClientError: ...
    def __getattr__(self, name: str) -> None: ...

class ClientExceptionsFactory:
    def __init__(self) -> None: ...
    def create_client_exceptions(self, service_model: ServiceModel) -> BaseClientExceptions: ...



"""
Type annotations for botocore.retries.throttling module.

Copyright 2025 Vlad Emelianov
"""

from typing import Any, NamedTuple

class CubicParams(NamedTuple):
    w_max: Any
    k: Any
    last_fail: Any

class CubicCalculator:
    def __init__(
        self,
        starting_max_rate: Any,
        start_time: Any,
        scale_constant: Any = ...,
        beta: Any = ...,
    ) -> None: ...
    def success_received(self, timestamp: Any) -> Any: ...
    def error_received(self, current_rate: Any, timestamp: Any) -> Any: ...
    def get_params_snapshot(self) -> Any: ...



"""
Type annotations for botocore.docs.service module.

Copyright 2025 Vlad Emelianov
"""

from botocore.docs.bcdoc.restdoc import DocumentStructure
from botocore.session import Session

class ServiceDocumenter:
    def __init__(self, service_name: str, session: Session, root_docs_path: str) -> None: ...
    def document_service(self) -> bytes: ...
    def title(self, section: DocumentStructure) -> None: ...
    def table_of_contents(self, section: DocumentStructure) -> None: ...
    def client_api(self, section: DocumentStructure) -> None: ...
    def client_exceptions(self, section: DocumentStructure) -> None: ...
    def paginator_api(self, section: DocumentStructure) -> None: ...
    def waiter_api(self, section: DocumentStructure) -> None: ...
    def get_examples(self, service_name: str, api_version: str | None = ...) -> list[str]: ...
    def client_context_params(self, section: DocumentStructure) -> None: ...
